@@ -6,15 +6,16 @@ __all__ = ['SafeStreamSocket', 'PlistSocket']
 
 import logging
 import os
+import plistlib
 import socket
 import ssl
 import struct
-import plistlib
 import threading
+import weakref
+from typing import Any, Union
 
-from typing import Union, Any
-from .exceptions import *
 from ._proto import PROGRAM_NAME
+from .exceptions import *
 
 logger = logging.getLogger(PROGRAM_NAME)
 
@@ -38,25 +39,35 @@ class SafeStreamSocket:
         self._sock = None
         if isinstance(addr, socket.socket):
             self._sock = addr
-            return
-        if isinstance(addr, SafeStreamSocket):  # copy self
-            self._sock = addr._sock
-            return
-
-        if isinstance(addr, str):
-            if ':' in addr:
-                host, port = addr.split(":", 1)
-                addr = (host, int(port))
-                family = socket.AF_INET
-            elif os.path.exists(addr):
-                family = socket.AF_UNIX
-            else:
-                raise MuxError("socket unix:{} unable to connect".format(addr))
         else:
-            family = socket.AF_INET
-        self._sock = socket.socket(family, socket.SOCK_STREAM)
-        self._sock.connect(addr)
-        
+            if isinstance(addr, str):
+                if ':' in addr:
+                    host, port = addr.split(":", 1)
+                    addr = (host, int(port))
+                    family = socket.AF_INET
+                elif os.path.exists(addr):
+                    family = socket.AF_UNIX
+                else:
+                    raise MuxError("socket unix:{} unable to connect".format(addr))
+            else:
+                family = socket.AF_INET
+            self._sock = socket.socket(family, socket.SOCK_STREAM)
+            self._sock.connect(addr)
+
+        self._finalizer = weakref.finalize(self, self._cleanup)
+    
+    def _cleanup(self):
+        logger.debug("Socket %d closed", self._id)
+        self._sock.close()
+        if hasattr(self, "_dup_sock"):
+            self._dup_sock.close()
+    
+    def close(self):
+        self._finalizer()
+    
+    @property
+    def closed(self) -> bool:
+        return not self._finalizer.alive
 
     @property
     def id(self) -> int:
@@ -94,18 +105,11 @@ class SafeStreamSocket:
         
         self._sock = ssock
 
-    def close(self):
-        logger.debug("Socket %d closed", self._id)
-        self._sock.close()
-
     def __enter__(self):
         return self
 
     def __exit__(self, *args):
         self.close()
-    
-    #def __del__(self):
-    #    self.close()
 
 
 class PlistSocket(SafeStreamSocket):
@@ -174,3 +178,24 @@ class PlistSocket(SafeStreamSocket):
     def send_recv_packet(self, payload: dict) -> dict:
         self.send_packet(payload)
         return self.recv_packet()
+
+
+class PlistSocketProperty:
+    def __init__(self, psock: PlistSocket):
+        self._psock = psock
+        self.prepare()
+    
+    @property
+    def psock(self) -> PlistSocket:
+        return self._psock
+    
+    def prepare(self):
+        pass
+
+    def send_packet(self, payload: dict, message_type: int = 8):
+        return self.psock.send_packet(payload, message_type)
+    
+    def recv_packet(self, header_size=None) -> dict:
+        return self.psock.recv_packet(header_size)
+    
+    
\ No newline at end of file
